@@ -1,9 +1,6 @@
 /* nodes.c -- how to get an Info file and node.
-   $Id: nodes.c 7831 2017-06-18 12:50:18Z gavin $
 
-   Copyright 1993, 1998, 1999, 2000, 2002, 2003, 2004, 2006, 2007,
-   2008, 2009, 2011, 2012, 2013, 2014, 2015, 2016, 2017 Free Software 
-   Foundation, Inc.
+   Copyright 1993-2022 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +22,8 @@
 #include "nodes.h"
 #include "search.h"
 #include "filesys.h"
-#include "info-utils.h"
+#include "scan.h"
+#include "util.h"
 #include "tag.h"
 #include "man.h"
 #include "variables.h"
@@ -129,6 +127,7 @@ build_tags_and_nodes (FILE_BUFFER *file_buffer)
         return;
 
       /* Skip "Indirect:" line. */
+      position += skip_node_separator (file_buffer->contents + position);
       position += strlen (INDIRECT_TABLE_LABEL);
       position += strspn (file_buffer->contents + position, "\r\n");
 
@@ -305,7 +304,10 @@ get_nodes_of_tags_table (FILE_BUFFER *file_buffer,
       for (p = 0; nodedef[p] && nodedef[p] != INFO_TAGSEP; p++)
         ;
       if (nodedef[p] != INFO_TAGSEP)
-        continue;
+        {
+          free (entry);
+          continue;
+        }
 
       entry->nodename = xmalloc (p + 1);
       strncpy (entry->nodename, nodedef, p);
@@ -360,33 +362,33 @@ get_tags_of_indirect_tags_table (FILE_BUFFER *file_buffer,
   /* We have the list of tags in file_buffer->tags.  Get the list of
      subfiles from the indirect table. */
   {
-    char *start, *end, *line;
-    SUBFILE *subfile;
+  char *start, *end, *line;
+  SUBFILE *subfile;
 
-    start = indirect_binding->buffer + indirect_binding->start;
-    end = indirect_binding->buffer + indirect_binding->end;
-    line = start;
+  start = indirect_binding->buffer + indirect_binding->start;
+  end = indirect_binding->buffer + indirect_binding->end;
+  line = start;
 
-    while (line < end)
-      {
-        int colon;
+  while (line < end)
+    {
+      int colon;
 
-        colon = string_in_line (":", line);
+      colon = string_in_line (":", line);
 
-        if (colon == -1)
-          break;
+      if (colon == -1)
+        break;
 
-        subfile = xmalloc (sizeof (SUBFILE));
-        subfile->filename = xmalloc (colon);
-        strncpy (subfile->filename, line, colon - 1);
-        subfile->filename[colon - 1] = 0;
-        subfile->first_byte = (long) atol (line + colon);
+      subfile = xmalloc (sizeof (SUBFILE));
+      subfile->filename = xmalloc (colon);
+      strncpy (subfile->filename, line, colon - 1);
+      subfile->filename[colon - 1] = 0;
+      subfile->first_byte = (long) atol (line + colon);
 
-        add_pointer_to_array (subfile, subfiles_index, subfiles, 
-                              subfiles_slots, 10);
+      add_pointer_to_array (subfile, subfiles_index, subfiles, 
+                            subfiles_slots, 10);
 
-        while (*line++ != '\n');
-      }
+      while (*line++ != '\n');
+    }
   }
 
   /* If we have successfully built the indirect files table, then
@@ -396,103 +398,104 @@ get_tags_of_indirect_tags_table (FILE_BUFFER *file_buffer,
       free_file_buffer_tags (file_buffer);
       return;
     }
-  else
+
+  {
+  int tags_index;
+  long header_length;
+  SEARCH_BINDING binding;
+
+  char *containing_dir;
+  char *temp;
+  int len_containing_dir;
+
+  /* Find the length of the header of the file containing the indirect
+     tags table.  This header appears at the start of every file.  We
+     want the absolute position of each node within each subfile, so
+     we subtract the start of the containing subfile from the logical
+     position of the node, and then add the length of the header in. */
+  binding.buffer = file_buffer->contents;
+  binding.start = 0;
+  binding.end = file_buffer->filesize;
+  binding.flags = S_FoldCase;
+
+  header_length = find_node_separator (&binding);
+  if (header_length == -1)
+    header_length = 0;
+
+  /* Build the file buffer's list of subfiles. */
+  containing_dir = xstrdup (file_buffer->fullpath);
+  temp = filename_non_directory (containing_dir);
+
+  if (temp > containing_dir)
     {
-      int tags_index;
-      long header_length;
-      SEARCH_BINDING binding;
-
-      /* Find the length of the header of the file containing the indirect
-         tags table.  This header appears at the start of every file.  We
-         want the absolute position of each node within each subfile, so
-         we subtract the start of the containing subfile from the logical
-         position of the node, and then add the length of the header in. */
-      binding.buffer = file_buffer->contents;
-      binding.start = 0;
-      binding.end = file_buffer->filesize;
-      binding.flags = S_FoldCase;
-
-      header_length = find_node_separator (&binding);
-      if (header_length == -1)
-        header_length = 0;
-
-      /* Build the file buffer's list of subfiles. */
-      {
-        char *containing_dir = xstrdup (file_buffer->fullpath);
-        char *temp = filename_non_directory (containing_dir);
-        int len_containing_dir;
-
-        if (temp > containing_dir)
-          {
-            if (HAVE_DRIVE (file_buffer->fullpath) &&
-                temp == containing_dir + 2)
-              {
-                /* Avoid converting "d:foo" into "d:/foo" below.  */
-                *temp = '.';
-                temp += 2;
-              }
-            temp[-1] = 0;
-          }
-
-        len_containing_dir = strlen (containing_dir);
-
-        for (i = 0; subfiles[i]; i++);
-
-        file_buffer->subfiles = xmalloc ((1 + i) * sizeof (char *));
-
-        for (i = 0; subfiles[i]; i++)
-          {
-            char *fullpath;
-
-            fullpath = xmalloc
-              (2 + strlen (subfiles[i]->filename) + len_containing_dir);
-
-            sprintf (fullpath, "%s/%s",
-                     containing_dir, subfiles[i]->filename);
-
-            file_buffer->subfiles[i] = fullpath;
-          }
-        file_buffer->subfiles[i] = NULL;
-        free (containing_dir);
-      }
-
-      /* For each node in the file's tags table, remember the starting
-         position. */
-      for (tags_index = 0; (entry = file_buffer->tags[tags_index]);
-           tags_index++)
+      if (HAVE_DRIVE (file_buffer->fullpath) &&
+          temp == containing_dir + 2)
         {
-          for (i = 0;
-               subfiles[i] && entry->nodestart >= subfiles[i]->first_byte;
-               i++);
-
-          /* If the Info file containing the indirect tags table is
-             malformed, then give up. */
-          if (!i)
-            {
-              /* The Info file containing the indirect tags table is
-                 malformed.  Give up. */
-              for (i = 0; subfiles[i]; i++)
-                {
-                  free (subfiles[i]->filename);
-                  free (subfiles[i]);
-                  free (file_buffer->subfiles[i]);
-                }
-              file_buffer->subfiles = NULL;
-              free_file_buffer_tags (file_buffer);
-              return;
-            }
-
-          /* SUBFILES[i] is the index of the first subfile whose logical
-             first byte is greater than the logical offset of this node's
-             starting position.  This means that the subfile directly
-             preceding this one is the one containing the node. */
-
-          entry->filename = file_buffer->subfiles[i - 1];
-          entry->nodestart -= subfiles[i - 1]->first_byte;
-          entry->nodestart += header_length;
+          /* Avoid converting "d:foo" into "d:/foo" below.  */
+          *temp = '.';
+          temp += 2;
         }
+      temp[-1] = 0;
     }
 
+  len_containing_dir = strlen (containing_dir);
+
+  for (i = 0; subfiles[i]; i++);
+  file_buffer->subfiles = xmalloc ((1 + i) * sizeof (char *));
+
+  for (i = 0; subfiles[i]; i++)
+    {
+      char *fullpath;
+      fullpath = xmalloc
+        (2 + strlen (subfiles[i]->filename) + len_containing_dir);
+
+      sprintf (fullpath, "%s/%s",
+               containing_dir, subfiles[i]->filename);
+
+      file_buffer->subfiles[i] = fullpath;
+    }
+  file_buffer->subfiles[i] = NULL;
+  free (containing_dir);
+
+  /* For each node in the file's tags table, remember the starting
+     position. */
+  for (tags_index = 0; (entry = file_buffer->tags[tags_index]);
+       tags_index++)
+    {
+      for (i = 0;
+           subfiles[i] && entry->nodestart >= subfiles[i]->first_byte;
+           i++);
+
+      /* If the Info file containing the indirect tags table is
+         malformed, then give up. */
+      if (!i)
+        {
+          /* The Info file containing the indirect tags table is
+             malformed.  Give up. */
+          for (i = 0; subfiles[i]; i++)
+            {
+              free (subfiles[i]->filename);
+              free (subfiles[i]);
+              free (file_buffer->subfiles[i]);
+            }
+          free (file_buffer->subfiles);
+          file_buffer->subfiles = NULL;
+          free_file_buffer_tags (file_buffer);
+          free (subfiles);
+          return;
+        }
+
+      /* SUBFILES[i] is the index of the first subfile whose logical
+         first byte is greater than the logical offset of this node's
+         starting position.  This means that the subfile directly
+         preceding this one is the one containing the node. */
+
+      entry->filename = file_buffer->subfiles[i - 1];
+      entry->nodestart -= subfiles[i - 1]->first_byte;
+      entry->nodestart += header_length;
+    }
+  }
+
   /* Free the structures assigned to SUBFILES.  Free the names as well
      as the structures themselves, then finally, the array. */
   for (i = 0; subfiles[i]; i++)
@@ -631,35 +634,6 @@ info_find_file (char *filename)
   else
     fullpath = xstrdup (filename);
 
-  /* FIXME: Put the following in info_find_fullpath, or remove
-     it altogether. */
-  /* If the file referenced by the name returned from info_find_fullpath ()
-     doesn't exist, then try again with the last part of the filename
-     appearing in lowercase. */
-  /* This is probably not needed at all on those systems which define
-     FILENAME_CMP to be mbscasecmp.  But let's do it anyway, lest some
-     network redirector supports case sensitivity.  */
-  if (!fullpath)
-    {
-      char *lowered_name;
-      char *tmp_basename;
-
-      lowered_name = xstrdup (filename);
-      tmp_basename = filename_non_directory (lowered_name);
-
-      while (*tmp_basename)
-        {
-          if (isupper (*tmp_basename))
-            *tmp_basename = tolower (*tmp_basename);
-
-          tmp_basename++;
-        }
-
-      fullpath = info_find_fullpath (lowered_name, 0);
-
-      free (lowered_name);
-    }
-
   /* If the file wasn't found, give up, returning a NULL pointer. */
   if (!fullpath)
     return NULL;
@@ -678,6 +652,7 @@ info_find_subfile (char *fullpath)
   char *with_extension = 0;
   int i;
   FILE_BUFFER *file_buffer = 0;
+  int fullpath_len = strlen (fullpath);
 
   /* First try to find the file in our list of already loaded files. */
   if (info_loaded_files)
@@ -685,7 +660,9 @@ info_find_subfile (char *fullpath)
       for (i = 0; (file_buffer = info_loaded_files[i]); i++)
         /* Check if fullpath starts the name of the recorded file (extra
            extensions like ".info.gz" could be added.) */
-        if (!strncmp (file_buffer->fullpath, fullpath, strlen (fullpath)))
+        if (!strncmp (file_buffer->fullpath, fullpath, fullpath_len)
+            && (file_buffer->fullpath[fullpath_len] == '\0'
+                || file_buffer->fullpath[fullpath_len] == '.'))
           {
             struct stat new_info, *old_info;
 
@@ -754,7 +731,18 @@ info_load_file (char *fullpath, int is_subfile)
   get_file_character_encoding (file_buffer);
 
   if (!is_subfile)
-    build_tags_and_nodes (file_buffer);
+    {
+      build_tags_and_nodes (file_buffer);
+      if (!file_buffer->tags)
+        {
+          free (file_buffer->fullpath);
+          free (file_buffer->filename);
+          free (file_buffer->contents);
+          free (file_buffer->encoding);
+          free (file_buffer);
+          return 0;
+        }
+    }
   else
     file_buffer->flags |= N_Subfile;
 
@@ -861,9 +849,6 @@ info_reload_file_buffer_contents (FILE_BUFFER *fb)
 /* Functions for node creation and retrieval. */
 
 static long get_node_length (SEARCH_BINDING *binding);
-static void get_filename_and_nodename (NODE *node,
-                                      char **filename, char **nodename,
-                                      char *filename_in, char *nodename_in);
 static void node_set_body_start (NODE *node);
 static int adjust_nodestart (FILE_BUFFER *file_buffer, TAG *tag);
 
@@ -943,8 +928,50 @@ info_get_node_with_defaults (char *filename_in, char *nodename_in,
 
   info_recent_file_error = NULL;
 
-  get_filename_and_nodename (defaults, &filename, &nodename,
-                             filename_in, nodename_in);
+  filename = filename_in;
+  if (filename_in)
+    {
+      filename = xstrdup (filename_in);
+      if (follow_strategy == FOLLOW_REMAIN
+          && defaults && defaults->fullpath
+          && filename_in)
+        {
+          /* Find the directory in the filename for defaults, and look in
+             that directory first. */
+          char *file_in_same_dir;
+          char saved_char, *p;
+
+          p = defaults->fullpath + strlen (defaults->fullpath);
+          while (p > defaults->fullpath && !IS_SLASH (*p))
+            p--;
+
+          if (p > defaults->fullpath)
+            {
+              saved_char = *p;
+              *p = 0;
+              file_in_same_dir = info_add_extension (defaults->fullpath,
+                                                     filename, 0);
+              *p = saved_char;
+
+              if (file_in_same_dir)
+                file_buffer = info_find_file (file_in_same_dir);
+              free (file_in_same_dir);
+            }
+        }
+    }
+  else
+    {
+      if (defaults)
+        filename = xstrdup (defaults->fullpath);
+      else
+        filename = xstrdup ("dir");
+    }
+
+  if (nodename_in && *nodename_in)
+    nodename = xstrdup (nodename_in);
+  else
+    /* If NODENAME is not specified, it defaults to "Top". */
+    nodename = xstrdup ("Top");
 
   /* If the file to be looked up is "dir", build the contents from all of
      the "dir"s and "localdir"s found in INFOPATH. */
@@ -960,33 +987,6 @@ info_get_node_with_defaults (char *filename_in, char *nodename_in,
       goto cleanup_and_exit;
     }
 
-
-  if (follow_strategy == FOLLOW_REMAIN
-      && defaults && defaults->fullpath)
-    {
-      /* Find the directory in the filename for defaults, and look in
-         that directory first. */
-      char *file_in_same_dir;
-      char saved_char, *p;
-
-      p = defaults->fullpath + strlen (defaults->fullpath);
-      while (p > defaults->fullpath && !IS_SLASH (*p))
-        p--;
-
-      if (p > defaults->fullpath)
-        {
-          saved_char = *p;
-          *p = 0;
-
-          file_in_same_dir = info_add_extension (defaults->fullpath,
-                                                 filename, 0);
-          if (file_in_same_dir)
-            file_buffer = info_find_file (file_in_same_dir);
-          free (file_in_same_dir);
-          *p = saved_char;
-        }
-    }
-
   if (!file_buffer)
     file_buffer = info_find_file (filename);
 
@@ -996,18 +996,6 @@ info_get_node_with_defaults (char *filename_in, char *nodename_in,
       node = info_get_node_of_file_buffer (file_buffer, nodename);
     }
 
-  if (!file_buffer)
-    {
-      /* Try to find a man page with this name as a fall back. */
-      node = get_manpage_node (filename);
-      if (!node)
-        {
-          if (filesys_error_number)
-            info_recent_file_error =
-              filesys_error_string (filename, filesys_error_number);
-        }
-    }
-
   /* If the node not found was "Top", try again with different case. */
   if (!node && (nodename && mbscasecmp (nodename, "Top") == 0))
     {
@@ -1031,33 +1019,6 @@ info_get_node (char *filename_in, char *nodename_in)
   return info_get_node_with_defaults (filename_in, nodename_in, 0);
 }
 
-/* Get filename and nodename of node to load using defaults from NODE.
-   Output values should be freed by caller. */
-static void
-get_filename_and_nodename (NODE *node,
-                           char **filename, char **nodename,
-                           char *filename_in, char *nodename_in)
-{
-  *filename = filename_in;
-
-  /* If FILENAME is not specified, it defaults to "dir". */
-  if (filename_in)
-    *filename = xstrdup (filename_in);
-  else
-    {
-      if (node)
-        *filename = xstrdup (node->fullpath);
-      else
-        *filename = xstrdup ("dir");
-    }
-
-  if (nodename_in && *nodename_in)
-    *nodename = xstrdup (nodename_in);
-  else
-    /* If NODENAME is not specified, it defaults to "Top". */
-    *nodename = xstrdup ("Top");
-}
-
 static void
 node_set_body_start (NODE *node)
 {
@@ -1327,8 +1288,7 @@ info_node_of_tag_ext (FILE_BUFFER *fb, TAG **tag_ptr, int fast)
              anchors that occur within the node. */
           scan_node_contents (node, parent, tag_ptr);
 
-          if (!preprocess_nodes_p)
-            node_set_body_start (node);
+          node_set_body_start (node);
           tag->cache = *node;
           if (!(node->flags & N_WasRewritten))
             tag->cache.contents = 0; /* Pointer into file buffer
