@@ -1,8 +1,6 @@
 /* window.c -- windows in Info.
-   $Id: window.c 7334 2016-09-03 16:47:27Z gavin $
 
-   Copyright 1993, 1997, 1998, 2001, 2002, 2003, 2004, 2007, 2008,
-   2011, 2012, 2013, 2014, 2015 Free Software Foundation, Inc.
+   Copyright 1993-2022 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +20,8 @@
 #include "info.h"
 #include "session.h"
 #include "display.h"
-#include "info-utils.h"
+#include "scan.h"
+#include "util.h"
 #include "doc.h"
 #include "tag.h"
 #include "variables.h"
@@ -415,9 +414,9 @@ window_change_window_height (WINDOW *window, int amount)
 
       /* If there aren't two neighboring windows, or if one of the neighbors
          is larger than the other one by at least AMOUNT, grow that one. */
-      if ((next && !prev) || ((next_avail - amount) >= prev_avail))
+      if (next_avail - amount >= prev_avail)
         grow_me_shrinking_next (window, next, amount);
-      else if ((prev && !next) || ((prev_avail - amount) >= next_avail))
+      else if (prev_avail - amount >= next_avail)
         grow_me_shrinking_prev (window, prev, amount);
       else
         {
@@ -575,10 +574,8 @@ window_set_node_of_window (WINDOW *window, NODE *node)
   calculate_line_starts (window);
   window_compute_line_map (window);
 
-  /* Clear displayed search matches if any.  TODO: do search again in new
-     node? */
-  free (window->matches);
-  window->matches = 0;
+  /* Clear displayed search matches if any. */
+  free_matches (&window->matches);
 
   window->flags |= W_UpdateWindow;
   if (node)
@@ -620,6 +617,8 @@ window_delete_window (WINDOW *window)
   free (window->log_line_no);
   free (window->line_map.map);
   free (window->modeline);
+  free_matches (&window->matches);
+  free (window->search_string);
 
   if (window == active_window)
     {
@@ -1175,7 +1174,9 @@ calculate_line_starts (WINDOW *win)
       size_t pbytes = 0; /* Not used. */
       int delim = 0;
 
-      printed_representation (&iter, &delim, pl_chars, &pchars, &pbytes);
+      /* Set pchars. */
+      (void) printed_representation (&iter, &delim, pl_chars,
+                                     &pchars, &pbytes);
 
       /* If this character can be printed without passing the width of
          the line, then include it in the line. */
@@ -1300,8 +1301,9 @@ window_compute_line_map (WINDOW *win)
       if (cur_ptr >= endp)
 	break;
       
-      printed_representation (&iter, &delim, win->line_map.used,
-                              &pchars, &pbytes);
+      /* Set pchars */
+      (void) printed_representation (&iter, &delim, win->line_map.used,
+                                     &pchars, &pbytes);
 
       while (pchars--)
         line_map_add (&win->line_map, cur_ptr - win->node->contents);
