@@ -1,5 +1,10 @@
 /* counter.h - definitions for counter.c */
-/* Copyright 2015-2019 Free Software Foundation, Inc.
+#ifndef COUNTER_H
+#define COUNTER_H
+
+#include "tree_types.h"
+
+/* Copyright 2015-2021 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,6 +19,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
+#include "tree_types.h"
+
 typedef struct {
     int *values; /* Array of values. */
     ELEMENT **elts; /* Elements corresponding to each value. */
@@ -26,3 +33,9 @@ void counter_pop (COUNTER *c);
 void counter_inc (COUNTER *c);
 void counter_dec (COUNTER *c);
 int counter_value (COUNTER *c, ELEMENT *e);
+
+/* A large positive number used to represent an unlimited number of remaining
+   arguments. */
+#define COUNTER_VARIADIC 32767
+
+#endif
