@@ -1,13 +1,13 @@
 use vars qw(%result_texis %result_texts %result_trees %result_errors 
    %result_indices %result_sectioning %result_nodes %result_menus
    %result_floats %result_converted %result_converted_errors 
-   %result_elements %result_directions_text);
+   %result_elements %result_directions_text %result_indices_sort_strings);
 
 use utf8;
 
 $result_trees{'lone_end_itemize'} = {
   'contents' => [],
-  'type' => 'text_root'
+  'type' => 'document_root'
 };
 
 $result_texis{'lone_end_itemize'} = '';
@@ -17,7 +17,7 @@ $result_texts{'lone_end_itemize'} = '';
 
 $result_errors{'lone_end_itemize'} = [
   {
-    'error_line' => ':1: unmatched `@end itemize\'
+    'error_line' => 'unmatched `@end itemize\'
 ',
     'file_name' => '',
     'line_nr' => 1,
@@ -28,4 +28,7 @@ $result_errors{'lone_end_itemize'} = [
 ];
 
 
+$result_floats{'lone_end_itemize'} = {};
+
+
 1;
